function [V,beta] =  projection(p,N,spec,varargin)
%
% This program takes as input a user-provided spectral density in the user-provided function, spec.
% The format of spec is, [spectraldensity] = spec(omega,.....), where .... are the arguments required
% to evaluate the spectrum.
%
% The program returns A1,  A2, ..., Ap in:
%
%               Y(t) = A1*Y(t-1) + A2*Y(t-2) + ... +   Ap*Y(t-p) + u(t), 
%
% where 
%                          beta = [A1 A2 ... Ap], Eu(t)u(t)'=V
%
% and p corresponds to the number of VAR lags, p.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%following are parameters that can be set by a user who would like to see
%some checks done on the operation of the program....

YWcheck=0;%this verifies that the Yule-Walker equations have been accurately solved
chkRieman=0;%this is some sort of check on the Riemann approximation to the integral
chkproj=0;

Notwo=N/2;

for iii = 1:Notwo+1
    omeg=2*pi*(iii-1)/N;
    [Spec]    =   feval(spec,omeg,varargin{:});
    Sspec(:,:,iii) = Spec; 
end
rowsJ=size(real(Sspec),1);

%We need to define C(0),C(1),...,C(p), the covariance functions
%of Y(t).  In the notation of the ACEL technical appendix C_tau(:,:,1)=C(0),
%,... C_tau(:,:,p+1) = C(p)

C_tau = zeros(rowsJ,rowsJ,p+1);

for kkk = 1:p+1
    %I am putting this since the imaginary part should be 0 in theory
    C_tau(:,:,kkk) = real(Sspec(:,:,1));
    for jjj = 2:Notwo
        w = (pi*(jjj-1))/Notwo;
        C_tau(:,:,kkk) = C_tau(:,:,kkk) + 2*real(Sspec(:,:,jjj)*exp(sqrt(-1)*w*(kkk-1)));
    end
    C_tau(:,:,kkk) = (C_tau(:,:,kkk) + real(Sspec(:,:,Notwo+1)*exp(sqrt(-1)*pi*(kkk-1))))/(2*Notwo);
end

%the covariances just computed integrated the suitably weighted spectrum
%the following check computes the spectrum from the covariances to verify
%that this first step was done to evaluate the accuracy of the Riemann
%approximation to the integral.
if chkRieman == 1
    checkRieman(p,rowsJ,Sspec,C_tau,Notwo,10^-10)
end

YuleWalkerX = zeros(p*rowsJ,p*rowsJ);
YuleWalkerd = zeros(rowsJ,p*rowsJ);

for kkk = 0:p-1
    for jjj = 0:p-1
        firstcol = rowsJ*kkk+1;
        lastcol = rowsJ*kkk+rowsJ;        
        firstrow = rowsJ*jjj+1;
        lastrow = rowsJ*jjj+rowsJ;
        if jjj > kkk
            YuleWalkerX(firstrow:lastrow,firstcol:lastcol)=C_tau(:,:,abs(jjj-kkk)+1)';
        else
            YuleWalkerX(firstrow:lastrow,firstcol:lastcol)=C_tau(:,:,abs(jjj-kkk)+1);
        end        
    end
    YuleWalkerd(:,firstcol:lastcol)=C_tau(:,:,kkk+2);
end
beta = YuleWalkerd/(YuleWalkerX);
%This part will write out VAR coefficients (beta)

if YWcheck == 1
    
    C_tau1check = zeros(rowsJ,rowsJ);
    C_tau1diff = zeros(rowsJ,rowsJ);
    for lll = 0:p-1
        if lll>0
            C_tau1check = C_tau1check + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,lll+1)';
        else 
            C_tau1check = C_tau1check + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,lll+1);
        end
    end
    C_tau1diff=C_tau1check - C_tau(:,:,2);
    
    
    %check second of the Yule Walker equations
    C_tau2check = zeros(rowsJ,rowsJ);
    C_tau2diff = zeros(rowsJ,rowsJ);
    for lll = 0:p-1
        if lll>1
            C_tau2check = C_tau2check + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,abs(lll-1)+1)';
        else 
            C_tau2check = C_tau2check + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,abs(lll-1)+1);
        end
    end
    C_tau2diff=C_tau2check - C_tau(:,:,3);
    
    %check last of the Yule Walker equations
    C_taupcheck = zeros(rowsJ,rowsJ);
    C_taupdiff = zeros(rowsJ,rowsJ);
    for lll = 0:p-1
        if lll>p-2
            C_taupcheck = C_taupcheck + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,abs(lll-(p-1))+1)';
        else 
            C_taupcheck = C_taupcheck + beta(:,rowsJ*lll+1:rowsJ*(lll+1))*C_tau(:,:,abs(lll-(p-1))+1);
        end
    end
    C_taupdiff=C_taupcheck - C_tau(:,:,p+1);
    
    YuleWalkerTol = max([max(max(abs(C_tau1diff))) max(max(abs(C_tau2diff))) max(max(abs(C_taupdiff)))]);
    if YuleWalkerTol > 10^-10
        fprintf('Max abs error of %15.12f in YW eqns exceeds tolerance of 1.0e-10',YuleWalkerTol); 
    end
    
end

V=C_tau(:,:,1);
nvar=size(V,1);
for ii = 1:p
    V=V-beta(:,(ii-1)*nvar+[1:nvar])*C_tau(:,:,ii+1)';
end